@@ -66,6 +66,7 @@ pub fn process(
     });
     let event_attributes = Attributes {
         client_id,
+        height: ctx.host_height(),
         ..Default::default()
     };
 
@@ -78,8 +79,8 @@ mod tests {
     use crate::prelude::*;
 
     use core::str::FromStr;
-    use tendermint::Time;
 
+    use crate::core::ics02_client::context::ClientReader;
     use crate::core::ics02_client::error::{Error, ErrorDetail};
     use crate::core::ics02_client::handler::dispatch;
     use crate::core::ics02_client::handler::ClientResult::Upgrade;
@@ -110,7 +111,7 @@ mod tests {
             signer,
         };
 
-        let output = dispatch(Time::now(), &ctx, ClientMsg::UpgradeClient(msg.clone()));
+        let output = dispatch(&ctx, ClientMsg::UpgradeClient(msg.clone()));
 
         match output {
             Ok(HandlerOutput {
@@ -121,8 +122,9 @@ mod tests {
                 assert_eq!(events.len(), 1);
                 let event = events.pop().unwrap();
                 assert!(
-                    matches!(event, IbcEvent::UpgradeClient(e) if e.client_id() == &msg.client_id)
+                    matches!(event, IbcEvent::UpgradeClient(ref e) if e.client_id() == &msg.client_id)
                 );
+                assert_eq!(event.height(), ctx.host_height());
                 assert!(log.is_empty());
                 // Check the result
                 match result {
@@ -155,7 +157,7 @@ mod tests {
             signer,
         };
 
-        let output = dispatch(Time::now(), &ctx, ClientMsg::UpgradeClient(msg.clone()));
+        let output = dispatch(&ctx, ClientMsg::UpgradeClient(msg.clone()));
 
         match output {
             Err(Error(ErrorDetail::ClientNotFound(e), _)) => {
@@ -183,7 +185,7 @@ mod tests {
             signer,
         };
 
-        let output = dispatch(Time::now(), &ctx, ClientMsg::UpgradeClient(msg.clone()));
+        let output = dispatch(&ctx, ClientMsg::UpgradeClient(msg.clone()));
 
         match output {
             Err(Error(ErrorDetail::LowUpgradeHeight(e), _)) => {
