@@ -8,10 +8,13 @@ use crate::core::ics02_client::client_consensus::AnyConsensusState;
 use crate::core::ics02_client::client_state::AnyClientState;
 use crate::core::ics03_connection::connection::ConnectionEnd;
 use crate::core::ics04_channel::channel::ChannelEnd;
+use crate::core::ics04_channel::handler::recv_packet::RecvPacketResult;
 use crate::core::ics04_channel::handler::{ChannelIdState, ChannelResult};
 use crate::core::ics04_channel::{error::Error, packet::Receipt};
-use crate::core::ics05_port::capabilities::Capability;
+use crate::core::ics05_port::capabilities::ChannelCapability;
+use crate::core::ics05_port::context::CapabilityReader;
 use crate::core::ics24_host::identifier::{ChannelId, ClientId, ConnectionId, PortId};
+use crate::core::ics26_routing::context::ModuleId;
 use crate::prelude::*;
 use crate::timestamp::Timestamp;
 use crate::Height;
@@ -19,7 +22,7 @@ use crate::Height;
 use super::packet::{PacketResult, Sequence};
 
 /// A context supplying all the necessary read-only dependencies for processing any `ChannelMsg`.
-pub trait ChannelReader {
+pub trait ChannelReader: CapabilityReader {
     /// Returns the ChannelEnd for the given `port_id` and `chan_id`.
     fn channel_end(&self, port_channel_id: &(PortId, ChannelId)) -> Result<ChannelEnd, Error>;
 
@@ -38,7 +41,7 @@ pub trait ChannelReader {
         height: Height,
     ) -> Result<AnyConsensusState, Error>;
 
-    fn authenticated_capability(&self, port_id: &PortId) -> Result<Capability, Error>;
+    fn authenticated_capability(&self, port_id: &PortId) -> Result<ChannelCapability, Error>;
 
     fn get_next_sequence_send(
         &self,
@@ -98,15 +101,18 @@ pub trait ChannelReader {
     /// Returns the maximum expected time per block
     fn max_expected_time_per_block(&self) -> Duration;
 
+    /// Calculates the block delay period using the connection's delay period and the maximum
+    /// expected time per block.
     fn block_delay(&self, delay_period_time: Duration) -> u64 {
-        let expected_time_per_block = self.max_expected_time_per_block();
-        if expected_time_per_block.is_zero() {
-            return 0;
-        }
-
-        FloatCore::ceil(delay_period_time.as_secs_f64() / expected_time_per_block.as_secs_f64())
-            as u64
+        calculate_block_delay(delay_period_time, self.max_expected_time_per_block())
     }
+
+    /// Return the module_id along with the capability associated with a given (channel-id, port_id)
+    fn lookup_module_by_channel(
+        &self,
+        channel_id: &ChannelId,
+        port_id: &PortId,
+    ) -> Result<(ModuleId, ChannelCapability), Error>;
 }
 
 /// A context supplying all the necessary write-only dependencies (i.e., storage writing facility)
@@ -115,7 +121,7 @@ pub trait ChannelKeeper {
     fn store_channel_result(&mut self, result: ChannelResult) -> Result<(), Error> {
         // The handler processed this channel & some modifications occurred, store the new end.
         self.store_channel(
-            (result.port_id.clone(), result.channel_id.clone()),
+            (result.port_id.clone(), result.channel_id),
             &result.channel_end,
         )?;
 
@@ -127,18 +133,12 @@ pub trait ChannelKeeper {
             // Associate also the channel end to its connection.
             self.store_connection_channels(
                 result.channel_end.connection_hops()[0].clone(),
-                &(result.port_id.clone(), result.channel_id.clone()),
+                &(result.port_id.clone(), result.channel_id),
             )?;
 
             // Initialize send, recv, and ack sequence numbers.
-            self.store_next_sequence_send(
-                (result.port_id.clone(), result.channel_id.clone()),
-                1.into(),
-            )?;
-            self.store_next_sequence_recv(
-                (result.port_id.clone(), result.channel_id.clone()),
-                1.into(),
-            )?;
+            self.store_next_sequence_send((result.port_id.clone(), result.channel_id), 1.into())?;
+            self.store_next_sequence_recv((result.port_id.clone(), result.channel_id), 1.into())?;
             self.store_next_sequence_ack((result.port_id, result.channel_id), 1.into())?;
         }
 
@@ -149,30 +149,34 @@ pub trait ChannelKeeper {
         match general_result {
             PacketResult::Send(res) => {
                 self.store_next_sequence_send(
-                    (res.port_id.clone(), res.channel_id.clone()),
+                    (res.port_id.clone(), res.channel_id),
                     res.seq_number,
                 )?;
 
                 self.store_packet_commitment(
-                    (res.port_id.clone(), res.channel_id.clone(), res.seq),
+                    (res.port_id.clone(), res.channel_id, res.seq),
                     res.timeout_timestamp,
                     res.timeout_height,
                     res.data,
                 )?;
             }
             PacketResult::Recv(res) => {
+                let res = match res {
+                    RecvPacketResult::Success(res) => res,
+                    RecvPacketResult::NoOp => unreachable!(),
+                };
                 match res.receipt {
                     None => {
                         // Ordered channel
                         self.store_next_sequence_recv(
-                            (res.port_id.clone(), res.channel_id.clone()),
+                            (res.port_id.clone(), res.channel_id),
                             res.seq_number,
                         )?
                     }
                     Some(r) => {
                         // Unordered channel
                         self.store_packet_receipt(
-                            (res.port_id.clone(), res.channel_id.clone(), res.seq),
+                            (res.port_id.clone(), res.channel_id, res.seq),
                             r,
                         )?
                     }
@@ -180,7 +184,7 @@ pub trait ChannelKeeper {
             }
             PacketResult::WriteAck(res) => {
                 self.store_packet_acknowledgement(
-                    (res.port_id.clone(), res.channel_id.clone(), res.seq),
+                    (res.port_id.clone(), res.channel_id, res.seq),
                     res.ack,
                 )?;
             }
@@ -194,7 +198,7 @@ pub trait ChannelKeeper {
                         //Unordered Channel
                         self.delete_packet_commitment((
                             res.port_id.clone(),
-                            res.channel_id.clone(),
+                            res.channel_id,
                             res.seq,
                         ))?;
                     }
@@ -203,13 +207,9 @@ pub trait ChannelKeeper {
             PacketResult::Timeout(res) => {
                 if let Some(c) = res.channel {
                     //Ordered Channel
-                    self.store_channel((res.port_id.clone(), res.channel_id.clone()), &c)?;
+                    self.store_channel((res.port_id.clone(), res.channel_id), &c)?;
                 }
-                self.delete_packet_commitment((
-                    res.port_id.clone(),
-                    res.channel_id.clone(),
-                    res.seq,
-                ))?;
+                self.delete_packet_commitment((res.port_id.clone(), res.channel_id, res.seq))?;
             }
         }
         Ok(())
@@ -279,3 +279,15 @@ pub trait ChannelKeeper {
     /// Should never fail.
     fn increase_channel_counter(&mut self);
 }
+
+pub fn calculate_block_delay(
+    delay_period_time: Duration,
+    max_expected_time_per_block: Duration,
+) -> u64 {
+    if max_expected_time_per_block.is_zero() {
+        return 0;
+    }
+
+    FloatCore::ceil(delay_period_time.as_secs_f64() / max_expected_time_per_block.as_secs_f64())
+        as u64
+}
