@@ -13,12 +13,12 @@ use crate::prelude::*;
 
 pub(crate) fn process(
     ctx: &dyn ChannelReader,
-    msg: MsgChannelOpenAck,
+    msg: &MsgChannelOpenAck,
 ) -> HandlerResult<ChannelResult, Error> {
     let mut output = HandlerOutput::builder();
 
     // Unwrap the old channel end and validate it against the message.
-    let mut channel_end = ctx.channel_end(&(msg.port_id.clone(), msg.channel_id.clone()))?;
+    let mut channel_end = ctx.channel_end(&(msg.port_id.clone(), msg.channel_id))?;
 
     // Validate that the channel end is in a state where it can be ack.
     if !channel_end.state_matches(&State::Init) && !channel_end.state_matches(&State::TryOpen) {
@@ -51,8 +51,7 @@ pub(crate) fn process(
     // Proof verification in two steps:
     // 1. Setup: build the Channel as we expect to find it on the other party.
 
-    let expected_counterparty =
-        Counterparty::new(msg.port_id.clone(), Some(msg.channel_id.clone()));
+    let expected_counterparty = Counterparty::new(msg.port_id.clone(), Some(msg.channel_id));
 
     let counterparty = conn.counterparty();
     let ccid = counterparty.connection_id().ok_or_else(|| {
@@ -70,7 +69,7 @@ pub(crate) fn process(
     );
 
     // set the counterparty channel id to verify against it
-    channel_end.set_counterparty_channel_id(msg.counterparty_channel_id.clone());
+    channel_end.set_counterparty_channel_id(msg.counterparty_channel_id);
 
     //2. Verify proofs
     verify_channel_proofs(
@@ -90,7 +89,7 @@ pub(crate) fn process(
 
     let result = ChannelResult {
         port_id: msg.port_id.clone(),
-        channel_id: msg.channel_id.clone(),
+        channel_id: msg.channel_id,
         channel_id_state: ChannelIdState::Reused,
         channel_cap,
         channel_end,
@@ -98,9 +97,14 @@ pub(crate) fn process(
 
     let event_attributes = Attributes {
         channel_id: Some(msg.channel_id),
+        height: ctx.host_height(),
         ..Default::default()
     };
-    output.emit(IbcEvent::OpenAckChannel(event_attributes.into()));
+    output.emit(IbcEvent::OpenAckChannel(
+        event_attributes
+            .try_into()
+            .map_err(|_| Error::missing_channel_id())?,
+    ));
 
     Ok(output.with_result(result))
 }
@@ -120,7 +124,8 @@ mod tests {
     use crate::core::ics03_connection::msgs::conn_open_try::MsgConnectionOpenTry;
     use crate::core::ics03_connection::version::get_compatible_versions;
     use crate::core::ics04_channel::channel::{ChannelEnd, Counterparty, State};
-    use crate::core::ics04_channel::handler::{channel_dispatch, ChannelResult};
+    use crate::core::ics04_channel::context::ChannelReader;
+    use crate::core::ics04_channel::handler::channel_dispatch;
     use crate::core::ics04_channel::msgs::chan_open_ack::test_util::get_dummy_raw_msg_chan_open_ack;
     use crate::core::ics04_channel::msgs::chan_open_ack::MsgChannelOpenAck;
     use crate::core::ics04_channel::msgs::chan_open_try::test_util::get_dummy_raw_msg_chan_open_try;
@@ -193,10 +198,7 @@ mod tests {
         let chan_end = ChannelEnd::new(
             State::Init,
             *msg_chan_try.channel.ordering(),
-            Counterparty::new(
-                msg_chan_ack.port_id.clone(),
-                Some(msg_chan_ack.channel_id.clone()),
-            ),
+            Counterparty::new(msg_chan_ack.port_id.clone(), Some(msg_chan_ack.channel_id)),
             connection_vec0.clone(),
             msg_chan_try.channel.version().clone(),
         );
@@ -204,10 +206,7 @@ mod tests {
         let failed_chan_end = ChannelEnd::new(
             State::Open,
             *msg_chan_try.channel.ordering(),
-            Counterparty::new(
-                msg_chan_ack.port_id.clone(),
-                Some(msg_chan_ack.channel_id.clone()),
-            ),
+            Counterparty::new(msg_chan_ack.port_id.clone(), Some(msg_chan_ack.channel_id)),
             connection_vec0,
             msg_chan_try.channel.version().clone(),
         );
@@ -230,7 +229,7 @@ mod tests {
                     .with_port_capability(msg_chan_ack.port_id.clone())
                     .with_channel(
                         msg_chan_ack.port_id.clone(),
-                        msg_chan_ack.channel_id.clone(),
+                        msg_chan_ack.channel_id,
                         failed_chan_end,
                     ),
                 msg: ChannelMsg::ChannelOpenAck(msg_chan_ack.clone()),
@@ -248,7 +247,7 @@ mod tests {
                     .with_connection(cid.clone(), conn_end.clone())
                     .with_channel(
                         msg_chan_ack.port_id.clone(),
-                        msg_chan_ack.channel_id.clone(),
+                        msg_chan_ack.channel_id,
                         chan_end.clone(),
                     ),
                 msg: ChannelMsg::ChannelOpenAck(msg_chan_ack.clone()),
@@ -265,7 +264,7 @@ mod tests {
                     .with_port_capability(msg_chan_ack.port_id.clone())
                     .with_channel(
                         msg_chan_ack.port_id.clone(),
-                        msg_chan_ack.channel_id.clone(),
+                        msg_chan_ack.channel_id,
                         chan_end.clone(),
                     ),
                 msg: ChannelMsg::ChannelOpenAck(msg_chan_ack.clone()),
@@ -279,7 +278,7 @@ mod tests {
                     .with_port_capability(msg_chan_ack.port_id.clone())
                     .with_channel(
                         msg_chan_ack.port_id.clone(),
-                        msg_chan_ack.channel_id.clone(),
+                        msg_chan_ack.channel_id,
                         chan_end.clone(),
                     ),
                 msg: ChannelMsg::ChannelOpenAck(msg_chan_ack.clone()),
@@ -296,7 +295,7 @@ mod tests {
                     .with_port_capability(msg_chan_ack.port_id.clone())
                     .with_channel(
                         msg_chan_ack.port_id.clone(),
-                        msg_chan_ack.channel_id.clone(),
+                        msg_chan_ack.channel_id,
                         chan_end,
                     ),
                 msg: ChannelMsg::ChannelOpenAck(msg_chan_ack),
@@ -307,27 +306,28 @@ mod tests {
         .collect();
 
         for test in tests {
-            let res = channel_dispatch(&test.ctx, test.msg.clone());
+            let res = channel_dispatch(&test.ctx, &test.msg);
             // Additionally check the events and the output objects in the result.
             match res {
-                Ok(proto_output) => {
+                Ok((proto_output, res)) => {
                     assert!(
                             test.want_pass,
                             "chan_open_ack: test passed but was supposed to fail for test: {}, \nparams {:?} {:?}",
                             test.name,
-                            test.msg.clone(),
+                            test.msg,
                             test.ctx.clone()
                         );
 
+                    let proto_output = proto_output.with_result(());
                     assert!(!proto_output.events.is_empty()); // Some events must exist.
 
                     // The object in the output is a ConnectionEnd, should have init state.
-                    let res: ChannelResult = proto_output.result;
                     //assert_eq!(res.channel_id, msg_chan_init.channel_id().clone());
                     assert_eq!(res.channel_end.state().clone(), State::Open);
 
                     for e in proto_output.events.iter() {
                         assert!(matches!(e, &IbcEvent::OpenAckChannel(_)));
+                        assert_eq!(e.height(), test.ctx.host_height());
                     }
                 }
                 Err(e) => {
