@@ -13,23 +13,23 @@ use crate::prelude::*;
 
 pub(crate) fn process(
     ctx: &dyn ChannelReader,
-    msg: MsgChannelOpenConfirm,
+    msg: &MsgChannelOpenConfirm,
 ) -> HandlerResult<ChannelResult, Error> {
     let mut output = HandlerOutput::builder();
 
     // Unwrap the old channel end and validate it against the message.
-    let mut channel_end = ctx.channel_end(&(msg.port_id().clone(), msg.channel_id().clone()))?;
+    let mut channel_end = ctx.channel_end(&(msg.port_id.clone(), msg.channel_id))?;
 
     // Validate that the channel end is in a state where it can be confirmed.
     if !channel_end.state_matches(&State::TryOpen) {
         return Err(Error::invalid_channel_state(
-            msg.channel_id().clone(),
+            msg.channel_id,
             channel_end.state,
         ));
     }
 
     // Channel capabilities
-    let channel_cap = ctx.authenticated_capability(&msg.port_id().clone())?;
+    let channel_cap = ctx.authenticated_capability(&msg.port_id)?;
 
     // An OPEN IBC connection running on the local (host) chain should exist.
     if channel_end.connection_hops().len() != 1 {
@@ -50,8 +50,7 @@ pub(crate) fn process(
     // Proof verification in two steps:
     // 1. Setup: build the Channel as we expect to find it on the other party.
 
-    let expected_counterparty =
-        Counterparty::new(msg.port_id().clone(), Some(msg.channel_id().clone()));
+    let expected_counterparty = Counterparty::new(msg.port_id.clone(), Some(msg.channel_id));
 
     let connection_counterparty = conn.counterparty();
     let ccid = connection_counterparty.connection_id().ok_or_else(|| {
@@ -70,11 +69,11 @@ pub(crate) fn process(
     //2. Verify proofs
     verify_channel_proofs(
         ctx,
-        msg.proofs().height(),
+        msg.proofs.height(),
         &channel_end,
         &conn,
         &expected_channel_end,
-        msg.proofs(),
+        &msg.proofs,
     )
     .map_err(Error::chan_open_confirm_proof_verification)?;
 
@@ -84,18 +83,23 @@ pub(crate) fn process(
     channel_end.set_state(State::Open);
 
     let result = ChannelResult {
-        port_id: msg.port_id().clone(),
-        channel_id: msg.channel_id().clone(),
+        port_id: msg.port_id.clone(),
+        channel_id: msg.channel_id,
         channel_id_state: ChannelIdState::Reused,
         channel_cap,
         channel_end,
     };
 
     let event_attributes = Attributes {
-        channel_id: Some(msg.channel_id().clone()),
+        channel_id: Some(msg.channel_id),
+        height: ctx.host_height(),
         ..Default::default()
     };
-    output.emit(IbcEvent::OpenConfirmChannel(event_attributes.into()));
+    output.emit(IbcEvent::OpenConfirmChannel(
+        event_attributes
+            .try_into()
+            .map_err(|_| Error::missing_channel_id())?,
+    ));
 
     Ok(output.with_result(result))
 }
@@ -114,7 +118,8 @@ mod tests {
     use crate::core::ics03_connection::msgs::test_util::get_dummy_raw_counterparty;
     use crate::core::ics03_connection::version::get_compatible_versions;
     use crate::core::ics04_channel::channel::{ChannelEnd, Counterparty, Order, State};
-    use crate::core::ics04_channel::handler::{channel_dispatch, ChannelResult};
+    use crate::core::ics04_channel::context::ChannelReader;
+    use crate::core::ics04_channel::handler::channel_dispatch;
     use crate::core::ics04_channel::msgs::chan_open_confirm::test_util::get_dummy_raw_msg_chan_open_confirm;
     use crate::core::ics04_channel::msgs::chan_open_confirm::MsgChannelOpenConfirm;
     use crate::core::ics04_channel::msgs::ChannelMsg;
@@ -157,8 +162,8 @@ mod tests {
             State::TryOpen,
             Order::default(),
             Counterparty::new(
-                msg_chan_confirm.port_id().clone(),
-                Some(msg_chan_confirm.channel_id().clone()),
+                msg_chan_confirm.port_id.clone(),
+                Some(msg_chan_confirm.channel_id),
             ),
             vec![conn_id.clone()],
             Version::default(),
@@ -169,10 +174,10 @@ mod tests {
             ctx: context
                 .with_client(&client_id, Height::new(0, client_consensus_state_height))
                 .with_connection(conn_id, conn_end)
-                .with_port_capability(msg_chan_confirm.port_id().clone())
+                .with_port_capability(msg_chan_confirm.port_id.clone())
                 .with_channel(
-                    msg_chan_confirm.port_id().clone(),
-                    msg_chan_confirm.channel_id().clone(),
+                    msg_chan_confirm.port_id.clone(),
+                    msg_chan_confirm.channel_id,
                     chan_end,
                 ),
             msg: ChannelMsg::ChannelOpenConfirm(msg_chan_confirm),
@@ -182,27 +187,28 @@ mod tests {
         .collect();
 
         for test in tests {
-            let res = channel_dispatch(&test.ctx, test.msg.clone());
+            let res = channel_dispatch(&test.ctx, &test.msg);
             // Additionally check the events and the output objects in the result.
             match res {
-                Ok(proto_output) => {
+                Ok((proto_output, res)) => {
                     assert!(
                             test.want_pass,
                             "chan_open_confirm: test passed but was supposed to fail for test: {}, \nparams {:?} {:?}",
                             test.name,
-                            test.msg.clone(),
+                            test.msg,
                             test.ctx.clone()
                         );
 
+                    let proto_output = proto_output.with_result(());
                     assert!(!proto_output.events.is_empty()); // Some events must exist.
 
                     // The object in the output is a ConnectionEnd, should have init state.
-                    let res: ChannelResult = proto_output.result;
                     //assert_eq!(res.channel_id, msg_chan_init.channel_id().clone());
                     assert_eq!(res.channel_end.state().clone(), State::Open);
 
                     for e in proto_output.events.iter() {
                         assert!(matches!(e, &IbcEvent::OpenConfirmChannel(_)));
+                        assert_eq!(e.height(), test.ctx.host_height());
                     }
                 }
                 Err(e) => {
