@@ -16,26 +16,26 @@ use crate::prelude::*;
 
 pub fn process(
     ctx: &dyn ChannelReader,
-    msg: MsgTimeoutOnClose,
+    msg: &MsgTimeoutOnClose,
 ) -> HandlerResult<PacketResult, Error> {
     let mut output = HandlerOutput::builder();
 
     let packet = &msg.packet;
 
     let source_channel_end =
-        ctx.channel_end(&(packet.source_port.clone(), packet.source_channel.clone()))?;
+        ctx.channel_end(&(packet.source_port.clone(), packet.source_channel))?;
 
     let _channel_cap = ctx.authenticated_capability(&packet.source_port)?;
 
     let counterparty = Counterparty::new(
         packet.destination_port.clone(),
-        Some(packet.destination_channel.clone()),
+        Some(packet.destination_channel),
     );
 
     if !source_channel_end.counterparty_matches(&counterparty) {
         return Err(Error::invalid_packet_counterparty(
             packet.destination_port.clone(),
-            packet.destination_channel.clone(),
+            packet.destination_channel,
         ));
     }
 
@@ -44,7 +44,7 @@ pub fn process(
     //verify the packet was sent, check the store
     let packet_commitment = ctx.get_packet_commitment(&(
         packet.source_port.clone(),
-        packet.source_channel.clone(),
+        packet.source_channel,
         packet.sequence,
     ))?;
 
@@ -57,10 +57,8 @@ pub fn process(
         return Err(Error::incorrect_packet_commitment(packet.sequence));
     }
 
-    let expected_counterparty = Counterparty::new(
-        packet.source_port.clone(),
-        Some(packet.source_channel.clone()),
-    );
+    let expected_counterparty =
+        Counterparty::new(packet.source_port.clone(), Some(packet.source_channel));
 
     let counterparty = connection_end.counterparty();
     let ccid = counterparty.connection_id().ok_or_else(|| {
@@ -79,11 +77,11 @@ pub fn process(
 
     verify_channel_proofs(
         ctx,
-        msg.proofs().height(),
+        msg.proofs.height(),
         &source_channel_end,
         &connection_end,
         &expected_channel_end,
-        &msg.proofs.clone(),
+        &msg.proofs,
     )?;
 
     let result = if source_channel_end.order_matches(&Order::Ordered) {
@@ -95,31 +93,31 @@ pub fn process(
         }
         verify_next_sequence_recv(
             ctx,
-            msg.proofs().height(),
+            msg.proofs.height(),
             &connection_end,
             packet.clone(),
             msg.next_sequence_recv,
-            &msg.proofs.clone(),
+            &msg.proofs,
         )?;
 
         PacketResult::Timeout(TimeoutPacketResult {
             port_id: packet.source_port.clone(),
-            channel_id: packet.source_channel.clone(),
+            channel_id: packet.source_channel,
             seq: packet.sequence,
             channel: Some(source_channel_end),
         })
     } else {
         verify_packet_receipt_absence(
             ctx,
-            msg.proofs().height(),
+            msg.proofs.height(),
             &connection_end,
             packet.clone(),
-            &msg.proofs.clone(),
+            &msg.proofs,
         )?;
 
         PacketResult::Timeout(TimeoutPacketResult {
             port_id: packet.source_port.clone(),
-            channel_id: packet.source_channel.clone(),
+            channel_id: packet.source_channel,
             seq: packet.sequence,
             channel: None,
         })
@@ -128,7 +126,7 @@ pub fn process(
     output.log("success: packet timeout ");
 
     output.emit(IbcEvent::TimeoutOnClosePacket(TimeoutOnClosePacket {
-        height: Default::default(),
+        height: ctx.host_height(),
         packet: packet.clone(),
     }));
 
@@ -192,7 +190,7 @@ mod tests {
             Order::Ordered,
             Counterparty::new(
                 packet.destination_port.clone(),
-                Some(packet.destination_channel.clone()),
+                Some(packet.destination_channel),
             ),
             vec![ConnectionId::default()],
             Version::ics20(),
@@ -244,7 +242,7 @@ mod tests {
                     )
                     .with_packet_commitment(
                         msg.packet.source_port.clone(),
-                        msg.packet.source_channel.clone(),
+                        msg.packet.source_channel,
                         msg.packet.sequence,
                         data,
                     ),
@@ -256,7 +254,7 @@ mod tests {
         .collect();
 
         for test in tests {
-            let res = process(&test.ctx, test.msg.clone());
+            let res = process(&test.ctx, &test.msg);
             // Additionally check the events and the output objects in the result.
             match res {
                 Ok(proto_output) => {
@@ -271,6 +269,7 @@ mod tests {
                     assert!(!proto_output.events.is_empty()); // Some events must exist.
                     for e in proto_output.events.iter() {
                         assert!(matches!(e, &IbcEvent::TimeoutOnClosePacket(_)));
+                        assert_eq!(e.height(), test.ctx.host_height());
                     }
                 }
                 Err(e) => {
