@@ -33,17 +33,6 @@ impl MsgChannelCloseConfirm {
             signer,
         }
     }
-
-    /// Getter: borrow the `port_id` from this message.
-    pub fn port_id(&self) -> &PortId {
-        &self.port_id
-    }
-    pub fn channel_id(&self) -> &ChannelId {
-        &self.channel_id
-    }
-    pub fn proofs(&self) -> &Proofs {
-        &self.proofs
-    }
 }
 
 impl Msg for MsgChannelCloseConfirm {
@@ -181,7 +170,7 @@ mod tests {
             Test {
                 name: "Correct channel identifier".to_string(),
                 raw: RawMsgChannelCloseConfirm {
-                    channel_id: "channelid34".to_string(),
+                    channel_id: "channel-34".to_string(),
                     ..default_raw_msg.clone()
                 },
                 want_pass: true,
@@ -198,7 +187,7 @@ mod tests {
                 name: "Bad channel, name too long".to_string(),
                 raw: RawMsgChannelCloseConfirm {
                     channel_id:
-                        "abcdefghiasdfadsfasdfgdfsadfasdasdfasdasdfasddsfasdfasdjklmnopqrstu"
+                        "channel-12839128379182739812739879"
                             .to_string(),
                     ..default_raw_msg.clone()
                 },
