@@ -349,6 +349,7 @@ impl IbcEvent {
             IbcEvent::NewBlock(bl) => bl.height(),
             IbcEvent::CreateClient(ev) => ev.height(),
             IbcEvent::UpdateClient(ev) => ev.height(),
+            IbcEvent::UpgradeClient(ev) => ev.height(),
             IbcEvent::ClientMisbehaviour(ev) => ev.height(),
             IbcEvent::OpenInitConnection(ev) => ev.height(),
             IbcEvent::OpenTryConnection(ev) => ev.height(),
@@ -365,6 +366,7 @@ impl IbcEvent {
             IbcEvent::WriteAcknowledgement(ev) => ev.height(),
             IbcEvent::AcknowledgePacket(ev) => ev.height(),
             IbcEvent::TimeoutPacket(ev) => ev.height(),
+            IbcEvent::TimeoutOnClosePacket(ev) => ev.height(),
             _ => unimplemented!(),
         }
     }
@@ -423,12 +425,12 @@ impl IbcEvent {
         }
     }
 
-    pub fn channel_attributes(&self) -> Option<&ChannelAttributes> {
+    pub fn channel_attributes(self) -> Option<ChannelAttributes> {
         match self {
-            IbcEvent::OpenInitChannel(ev) => Some(ev.attributes()),
-            IbcEvent::OpenTryChannel(ev) => Some(ev.attributes()),
-            IbcEvent::OpenAckChannel(ev) => Some(ev.attributes()),
-            IbcEvent::OpenConfirmChannel(ev) => Some(ev.attributes()),
+            IbcEvent::OpenInitChannel(ev) => Some(ev.into()),
+            IbcEvent::OpenTryChannel(ev) => Some(ev.into()),
+            IbcEvent::OpenAckChannel(ev) => Some(ev.into()),
+            IbcEvent::OpenConfirmChannel(ev) => Some(ev.into()),
             _ => None,
         }
     }
@@ -463,21 +465,21 @@ impl IbcEvent {
     }
 }
 
-#[derive(Debug, Clone, Deserialize, Serialize)]
-pub struct RawObject {
+#[derive(Debug, Clone, Serialize)]
+pub struct RawObject<'a> {
     pub height: Height,
     pub action: String,
     pub idx: usize,
-    pub events: HashMap<String, Vec<String>>,
+    pub events: &'a HashMap<String, Vec<String>>,
 }
 
-impl RawObject {
+impl<'a> RawObject<'a> {
     pub fn new(
         height: Height,
         action: String,
         idx: usize,
-        events: HashMap<String, Vec<String>>,
-    ) -> RawObject {
+        events: &'a HashMap<String, Vec<String>>,
+    ) -> RawObject<'a> {
         RawObject {
             height,
             action,
@@ -500,7 +502,7 @@ pub fn extract_events(
     Err(Error::incorrect_event_type(action_string.to_string()))
 }
 
-pub fn extract_attribute(object: &RawObject, key: &str) -> Result<String, Error> {
+pub fn extract_attribute(object: &RawObject<'_>, key: &str) -> Result<String, Error> {
     let value = object
         .events
         .get(key)
@@ -510,6 +512,6 @@ pub fn extract_attribute(object: &RawObject, key: &str) -> Result<String, Error>
     Ok(value)
 }
 
-pub fn maybe_extract_attribute(object: &RawObject, key: &str) -> Option<String> {
+pub fn maybe_extract_attribute(object: &RawObject<'_>, key: &str) -> Option<String> {
     object.events.get(key).map(|tags| tags[object.idx].clone())
 }
