@@ -6,7 +6,6 @@
 #![allow(clippy::large_enum_variant)]
 #![deny(
     warnings,
-    // missing_docs,
     trivial_casts,
     trivial_numeric_casts,
     unused_import_braces,
@@ -46,6 +45,8 @@
 //! [relayer-repo]: https://github.com/informalsystems/ibc-rs/tree/master/relayer
 
 extern crate alloc;
+
+#[cfg(feature = "std")]
 extern crate std;
 
 mod prelude;
