@@ -1,6 +1,6 @@
 use crate::core::ics24_host::error::ValidationError;
 use crate::prelude::*;
-use prost_types::Any;
+use ibc_proto::google::protobuf::Any;
 
 pub trait Msg: Clone {
     type ValidationError;
